﻿Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False
    Friend FileName As String
    Friend SaveLoadFolder As String

    ' =======================================================================================
    '  FORM
    ' =======================================================================================

    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private sw As Integer
    Private sh As Integer
    Private Sub GetMaxScreenBounds()
        ' -------------------------------------- Gets an array of all the screens connected to the system.
        Dim Screens() As System.Windows.Forms.Screen = System.Windows.Forms.Screen.AllScreens
        ' -------------------------------------- For each screen
        Dim Index As Integer
        For Index = 0 To Screens.GetUpperBound(0)
            sw = Math.Max(sw, Screens(Index).Bounds.Left + Screens(Index).Bounds.Width)
            sh = Math.Max(sh, Screens(Index).Bounds.Top + Screens(Index).Bounds.Height)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, sw)           ' not more than dimensions of a maximized window
    '        .Height = Math.Min(.Height, sh)         ' not more than dimensions of a maximized window
    '        .Width = Math.Max(.Width, 320)          ' at least 320x240
    '        .Height = Math.Max(.Height, 240)        ' at least 320x240
    '        .Left = Math.Min(.Left, sw - .Width)    ' not beyond the right border
    '        .Top = Math.Min(.Top, sh - .Height)     ' not beyond the bottom border
    '        .Left = Math.Max(.Left, 0)              ' at least at the left border
    '        .Top = Math.Max(.Top, 0)                ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, sw)               ' not more than the VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, sh)             ' not more than the VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)               ' at least 32x24
            .Height = Math.Max(.Height, 24)             ' at least 32x24
            .Left = Math.Min(.Left, sw - 50)            ' not beyond the right border - 50 pixels
            .Top = Math.Min(.Top, sh - 50)              ' not beyond the bottom border - 50 pixels
            .Left = Math.Max(.Left, 100 - .Width)       ' at least at the left border + 50 pixels
            .Top = Math.Max(.Top, -10)                  ' at least at the top border
        End With
    End Sub


    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function

    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================

    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(Replace(l, ",", ".")))
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(Replace(l, ",", "."))
    End Function

    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(Replace(l, ",", ".")))
    End Function


    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function




    ' ================================================================== MENU
    Friend Sub LoadFile()
        Dim OFD As System.Windows.Forms.OpenFileDialog
        OFD = New System.Windows.Forms.OpenFileDialog()
        PrepareSaveLoadFolder()
        OFD.InitialDirectory = SaveLoadFolder
        OFD.FileName = FileName
        'OFD.RestoreDirectory = True
        OFD.DefaultExt = ".txt"
        OFD.AutoUpgradeEnabled = True
        OFD.Title = "Load script file"
        OFD.ValidateNames = False
        OFD.Filter = "Led Param Files ( .txt )|*.txt|All Files|*.*"
        If OFD.ShowDialog() = DialogResult.OK Then
            FileName = IO.Path.GetFileName(OFD.FileName)
            EventsAreEnabled = False
            LoadParams()
            EventsAreEnabled = True
        End If
    End Sub
    Friend Sub SaveFileAs()
        Dim SFD As System.Windows.Forms.SaveFileDialog
        SFD = New System.Windows.Forms.SaveFileDialog()
        PrepareSaveLoadFolder()
        SFD.InitialDirectory = SaveLoadFolder
        SFD.FileName = FileName
        'SFD.RestoreDirectory = False
        SFD.DefaultExt = ".txt"
        SFD.AutoUpgradeEnabled = True
        SFD.Title = "Save script file"
        SFD.ValidateNames = False
        SFD.Filter = "Led Param Files ( .txt )|*.txt|All Files|*.*"
        If SFD.ShowDialog() = DialogResult.OK Then
            FileName = IO.Path.GetFileName(SFD.FileName)
            If Not FileName.EndsWith(".txt", StringComparison.InvariantCultureIgnoreCase) Then FileName &= ".txt"
            SaveParams()
        End If
    End Sub

    Private Sub PrepareSaveLoadFolder()
        If My.Computer.FileSystem.FileExists(FileName) Then
            SaveLoadFolder = File_GetPath(FileName)
        End If
        If Not FolderExists(SaveLoadFolder) Then
            SaveLoadFolder = Application.StartupPath & "\Docs\"
        End If
    End Sub

    Public Function File_GetPath(ByVal str As String) As String
        Try
            Return IO.Path.GetDirectoryName(str) & "\"
        Catch
            Return str
        End Try
    End Function

    Public Function FolderExists(ByVal FolderName As String) As Boolean
        If FolderName Is Nothing Then Return False
        If FolderName.Length < 2 Then Return False
        FolderName = LCase(FolderName)
        Select Case FolderName
            Case "a:\", "b:\", "c:\", "d:\", "e:\", "f:\", "g:\", "h:\", "i:\", "j:\", "k:\"
                Return True
        End Select
        Return My.Computer.FileSystem.DirectoryExists(FolderName)
    End Function



    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        Try
            '
            Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
            Dim f As System.IO.StreamWriter
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("FileName", FileName))
            '
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            '
            f.Close()
        Catch
        End Try
    End Sub

    Friend Sub Load_INI()
        ' ------ defaults -------
        '
        ' -----------------------
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "FileName" : FileName = l
                        '
                    Case "Form1_Top" : Form1.Top = CInt(Val(l))
                    Case "Form1_Left" : Form1.Left = CInt(Val(l))
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub


    ' ==================================================================================================
    '  SAVE LOAD -- PARAMS
    ' ==================================================================================================

    Friend Sub SaveParams()
        '
        'If FileName = "" Then Exit Sub
        '
        Try
            Dim f As Int32 = FreeFile()
            '
            FileOpen(f, FileName, OpenMode.Output, OpenAccess.Write)
            PrintLine(f, " Program Params")
            PrintLine(f, "===========================================")
            'PrintLine(f, "FileName=", TAB(22), FileName)
            '
            PrintLine(f, "")
            PrintLine(f, " Led Params")
            PrintLine(f, "===========================================")
            PrintLine(f, "LedBeamAngle=", TAB(22), Form1.txt_LedBeamAngle.NumericValue.ToString)
            PrintLine(f, "LedMilliCandles=", TAB(22), Form1.txt_LedMilliCandles.NumericValue.ToString)
            PrintLine(f, "LedVoltage=", TAB(22), Form1.txt_LedVoltage.NumericValue.ToString)
            PrintLine(f, "LedCurrent=", TAB(22), Form1.txt_LedCurrent.NumericValue.ToString)
            '
            PrintLine(f, "")
            PrintLine(f, " Array Params")
            PrintLine(f, "===========================================")
            PrintLine(f, "ArrayResistors=", TAB(22), Form1.txt_ArrayResistors.NumericValue.ToString)
            PrintLine(f, "ArrayVoltageDrop=", TAB(22), Form1.txt_ArrayVoltageDrop.NumericValue.ToString)
            PrintLine(f, "ArrayRows=", TAB(22), Form1.txt_ArrayRows.NumericValue.ToString)
            PrintLine(f, "ArrayColumns=", TAB(22), Form1.txt_ArrayColumns.NumericValue.ToString)
            PrintLine(f, "ArraySupplyVoltage=", TAB(22), Form1.txt_ArraySupplyVoltage.NumericValue.ToString)
            PrintLine(f, "ArrayBeamAngle=", TAB(22), Form1.txt_ArrayBeamAngle.NumericValue.ToString)
            '
            FileClose(f)
            '
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Exclamation)
        Finally
            Form1.Text = "Led Array Calculator - " & IO.Path.GetFileNameWithoutExtension(FileName)
        End Try
    End Sub


    Friend Sub LoadParams()
        Dim l As String
        PrepareSaveLoadFolder()
        My.Computer.FileSystem.CurrentDirectory = SaveLoadFolder
        If My.Computer.FileSystem.FileExists(FileName) Then
            Dim f As Int32 = FreeFile()
            FileOpen(f, FileName, OpenMode.Input, OpenAccess.Read, OpenShare.Shared)
            Do While Not EOF(f)
                l = LineInput(f)
                Select Case ExtractParamName(l)
                    'Case "FileName" : FileName = l
                    ' -------------------------------------------------
                    Case "LedBeamAngle" : Form1.txt_LedBeamAngle.NumericValue = Val_Single(l)
                    Case "LedMilliCandles" : Form1.txt_LedMilliCandles.NumericValue = Val_Single(l)
                    Case "LedVoltage" : Form1.txt_LedVoltage.NumericValue = Val_Single(l)
                    Case "LedCurrent" : Form1.txt_LedCurrent.NumericValue = Val_Single(l)
                        '
                    Case "ArrayResistors" : Form1.txt_ArrayResistors.NumericValue = Val_Single(l)
                    Case "ArrayVoltageDrop" : Form1.txt_ArrayVoltageDrop.NumericValue = Val_Single(l)
                    Case "ArrayRows" : Form1.txt_ArrayRows.NumericValue = Val_Single(l)
                    Case "ArrayColumns" : Form1.txt_ArrayColumns.NumericValue = Val_Single(l)
                    Case "ArraySupplyVoltage" : Form1.txt_ArraySupplyVoltage.NumericValue = Val_Single(l)
                    Case "ArrayBeamAngle" : Form1.txt_ArrayBeamAngle.NumericValue = Val_Single(l)
                        '
                End Select
            Loop
            FileClose(f)
        End If
        '
        Form1.Text = "Theremino - Led Array Calculator - " & IO.Path.GetFileNameWithoutExtension(FileName)
        Form1.Calc_Led()
        Form1.Calc_Array()
    End Sub


End Module
